//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////

#include "appwindow.h"

#include <qapplication.h>
#include <qcanvas.h>
#include <qfiledialog.h>
#include <qmainwindow.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qstatusbar.h>
#include <qstring.h>
#include <qcolordialog.h>

#include "v4lcamera.h"
#include "campropdlg.h"
#include "imagepropdlg.h"
#include "appdata.h"
#include "ctcanvasitem.h"
#include "ctcanvasview.h"

#include <iostream>
using std::cout;
using std::endl;


extern AppData gAppData;


AppWindow::AppWindow() : QMainWindow( NULL, "app window", 0 )
{
   createMenuBarItems();

   createCanvas();

   statusBar()->message( "Ready" );

   m_last_sel_color.setRgb( 0, 0, 255 );
}


void AppWindow::createMenuBarItems()
{
   QPopupMenu* file_menu = new QPopupMenu( this );
   menuBar()->insertItem( "&File", file_menu );

   file_menu->insertItem( "&Open image...",
                          this,
                          SLOT( openFile() ),
                          CTRL + Key_O );

   file_menu->insertItem( "&Quit",
                          qApp,
                          SLOT( closeAllWindows() ),
                          CTRL + Key_Q );

   QPopupMenu* camera_menu = new QPopupMenu( this );
   menuBar()->insertItem( "&Camera", camera_menu );

   camera_menu->insertItem( "&Properties",
                            this,
                            SLOT( showCameraProps() ),
                            CTRL + Key_P );
   camera_menu->insertItem( "&Image Properties",
                            this,
                            SLOT( showImageProps() ),
                            CTRL + Key_I );

   QPopupMenu* imgproc_menu = new QPopupMenu( this );
   menuBar()->insertItem( "&Processing", imgproc_menu );
   imgproc_menu->insertItem( "Set &Target Color",
                             this,
                             SLOT( showSelectColor() ),
                             CTRL + Key_T );
}


void AppWindow::createCanvas()
{
   m_canvas = new QCanvas( this );
   m_canvas->resize( 356, 292 );

   m_canvas_view = new CTCanvasView( this );
   m_canvas_view->setCanvas( m_canvas );

   m_ctcanvas = new CTCanvasItem( m_canvas );
   m_ctcanvas->show();

   setCentralWidget( m_canvas_view );

   m_canvas->setUpdatePeriod( 20 );
   
   m_canvas_view->show();
}

void AppWindow::imageProcessed( Image* img, Rect* rect )
{
   m_ctcanvas->imageProcessed( img, rect );
}


///////////////////////////////////////////////////////////////////////////
// Public Slots
///////////////////////////////////////////////////////////////////////////

void AppWindow::openFile()
{
   QString filename = QFileDialog::getOpenFileName( QString::null,
                                                    "png (*.png)",
                                                    this,
                                                    "file open",
                                                    "Open PNG File" );

   if( filename != QString::null )
   {

      statusBar()->message( filename );
   }
}

void AppWindow::showCameraProps()
{
   CamPropDlg* prop_dlg = new CamPropDlg( this, gAppData.getCamera() );
   prop_dlg->exec();
   delete prop_dlg;
}

void AppWindow::showImageProps()
{
   ImagePropDlg* prop_dlg = new ImagePropDlg( this, gAppData.getCamera() );
   prop_dlg->exec();
   delete prop_dlg;
}

void AppWindow::showSelectColor()
{
   QColor selected_color;

   selected_color = QColorDialog::getColor( m_last_sel_color, this );

   if( selected_color.isValid() )
   {
      m_last_sel_color = selected_color;

      RGBPixel rgbpixel;

      rgbpixel.setRGB( qRed( selected_color.rgb() ),
                       qGreen( selected_color.rgb() ),
                       qBlue( selected_color.rgb() ) );

      m_ctimgproc->setTargetPixelColor( rgbpixel );
   }
}
