//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////

#include "ctcanvasitem.h"


#include <qcanvas.h>
#include <qpainter.h>
#include <qimage.h>
#include <qpixmap.h>

#include <sys/ioctl.h>
#include <sys/mman.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#include "image.h"

#define DEFAULT_IMG_WIDTH       356
#define DEFAULT_IMG_HEIGHT      292

const int CTCanvasItem::RTTI = 9999;

CTCanvasItem::CTCanvasItem( QCanvas* canvas ) : QCanvasPolygon( canvas ),
                                                  m_qimage()
{
   m_qimage.create( DEFAULT_IMG_WIDTH, DEFAULT_IMG_HEIGHT, 32 );

   QPointArray points;
   points.putPoints( 0, 4,
                     0,                 0,
                     m_qimage.width(),  0,
                     m_qimage.width(),  m_qimage.height(),
                     0,                 m_qimage.height() );

   setPoints( points );

   m_last_img = NULL;

   m_controller = NULL;
   m_imgproc = NULL;
   
   m_draw_rect = false;
}

CTCanvasItem::~CTCanvasItem()
{
}

void CTCanvasItem::imageProcessed( Image* img, Rect* rect )
{
   RGBPixel* img_buffer = img->scanLine( 0 );

   m_last_img = img;

   int pixel_ind;
   for( int row = 0; row < m_qimage.height(); row++ )
   {
      QRgb* rgb_line = (QRgb*) m_qimage.scanLine( row );

      pixel_ind = row * img->width();

      for( int col = 0; col < m_qimage.width(); col++ )
      {
         rgb_line[col] = qRgb( img_buffer[pixel_ind].r,
                               img_buffer[pixel_ind].g,
                               img_buffer[pixel_ind].b );

         pixel_ind++;
      }
   }

   if( rect != NULL )
   {
      m_qrect = QRect( rect->x1 - 5,
                       rect->y1 - 5,
                       rect->x2 - rect->x1 + 5,
                       rect->y2 - rect->y1 + 5 );
      m_draw_rect = true;
   }
   else
   {
      m_draw_rect = false;
   }
   
   canvas()->setAllChanged();
}

void CTCanvasItem::drawShape( QPainter& painter )
{
   painter.drawImage( 0, 0, m_qimage );

   if( m_draw_rect )
   {
      painter.setPen( Qt::green );
      painter.drawRect( m_qrect );
   }
}

void CTCanvasItem::mousePressEvent( QMouseEvent* e )
{
   if( m_last_img == NULL )
      return;

   Rect rect;
   rect.x1 = e->x() - 2;
   rect.x2 = rect.x1 + 2;
   rect.y1 = e->y() - 2;
   rect.y2 = rect.y1 + 2;

   //m_controller->imageProcessed( m_last_img, &rect );

   if( rect.x1 < 0 )
   {
      rect.x2 += -rect.x1;
      rect.x1 = 0;
   }
   else if( rect.x2 > m_qimage.width() )
   {
      rect.x1 -= (rect.x2 - m_qimage.width());
      rect.x2 = m_qimage.width();
   }

   if( rect.y1 < 0 )
   {
      rect.y2 += -rect.y1;
      rect.y1 = 0;
   }
   else if( rect.y2 > m_qimage.height() )
   {
      rect.y1 -= (rect.y2 - m_qimage.height());
      rect.y2 = m_qimage.height();
   }

   int r=0, g=0, b=0;
   QRgb pixel;

   for( int x = 0; x < 5; x++ )
   {
      for( int y = 0; y < 5; y++ )
      {
         pixel = m_qimage.pixel( rect.x1+x, rect.y1+y );
         r += qRed(pixel);
         g += qGreen(pixel);
         b += qBlue(pixel);
      }
   }

   RGBPixel rgbpixel;
   rgbpixel.r = (uchar)(r / 25);
   rgbpixel.g = (uchar)(g / 25);
   rgbpixel.b = (uchar)(b / 25);

   //printf("%3d %3d %3d\n", rgbpixel.r, rgbpixel.g, rgbpixel.b);
   
   //m_imgproc->setTargetPixelColor( rgbpixel );
}

