//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////

#include "ctcontroller.h"
#include "lnpctl.h"

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <pthread.h>


void* send_msg_start(void* cmd);

CTController::CTController()
{
   m_fov = 45.0;
   m_tan_half_fov = tan( (M_PI / 180.0) * m_fov / 2.0 );

   init_lnp();
}

CTController::~CTController()
{
}

void CTController::imageProcessed( Image* img, Rect* rect )
{
   if( rect == NULL || img == NULL )
      return;

   int x = (rect->x2 + rect->x1) / 2;
   int width = (int) img->width();

   int angle_from_center = calcAngleFromCenter( x, width );

   if( angle_from_center < 0 )
      sendCommand( LEFT, -angle_from_center );

   else if( angle_from_center > 0 )
      sendCommand( RIGHT, angle_from_center );

   else
      sendCommand( STOP, 0 );

}


int CTController::calcAngleFromCenter( int x, int width )
{
   float x_pct = 2 * (((float) x / (float) width) - 0.5);

   float angle = (180.0 / M_PI) * atan( x_pct * m_tan_half_fov );

   return (int) angle;
}


void CTController::sendCommand(int cmd, int angle)
{
   // start the communication to RCX in its own thread (problems otherwise)
   pthread_t send_msg_thread;

   char* cmd_byte = new char [1];
   cmd_byte[0] = (char) cmd;
   pthread_create( &send_msg_thread, NULL, send_msg_start, (void*) cmd_byte );

   /*if( cmd == LEFT )
      printf("Left  %2d\n", angle);
   else if( cmd == RIGHT )
      printf("Right %2d\n", angle);
   else if( cmd == STOP )
      printf("Stop\n");*/
}

void* send_msg_start(void* cmd)
{
   char* cmd_array = (char*) cmd;
   send_msg( cmd_array[0] );

   delete[] cmd_array;
   return NULL;
}
