//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////

#include "ctimgproc.h"

#include "types.h"

#define WORST_COLOR_MATCH       3.0


CTImgProc::Hsv CTImgProc::Rgb::toHSV()
{
   Hsv hsv;

   float min, max;

   min = r;
   if( g < min )
      min = g;
   if( b < min )
      min = b;

   max = r;
   if( g > max )
      max = g;
   if( b > max )
      max = b;

   hsv.v = max;

   hsv.s = (max != 0) ? ((max - min) / max) : 0;

   if( hsv.s == 0 )
      hsv.h = 0;
   else
   {
      float delta = max - min;

      if( r == max )
         hsv.h = (g - b) / delta;
      else if( g == max )
         hsv.h = 2.0 + (b - r) / delta;
      else
         hsv.h = 4.0 + (r - g) / delta;

      hsv.h = hsv.h * 60.0;

      if( hsv.h < 0.0 )
         hsv.h = hsv.h + 360.0;
   }

   return hsv;
}


CTImgProc::CTImgProc() : m_blur_filter( 3 ), m_prev_img(), m_avg_img()
{
   m_window_size = 1;

   m_target_color.r = 0;
   m_target_color.g = 0;
   m_target_color.b = 1.0;

   m_target_hsv = m_target_color.toHSV();
}

CTImgProc::~CTImgProc()
{
}

void CTImgProc::setTargetPixelColor( const RGBPixel& rgbpixel )
{
   m_target_color.r = (uchar)(rgbpixel.r / 255.0);
   m_target_color.g = (uchar)(rgbpixel.g / 255.0);
   m_target_color.b = (uchar)(rgbpixel.b / 255.0);

   m_target_hsv = m_target_color.toHSV();
}


void CTImgProc::frameGrabbed( Image* img )
{
   if( img == NULL )
      return;

   Rect rect;

   /*
   if( m_avg_img.width() != img->width() )
   {
      m_avg_img.create( img->width(), img->height() );

      if( m_prev_img.width() != img->width() )
         m_prev_img.create( img->width(), img->height() );
      img->copyToImage( m_prev_img );
      return;
   }

   for( uint x=0; x<img->width(); x++ )
   {
      for( uint y=0; y<img->height(); y++ )
      {
         RGBPixel p = img->pixel( y, x );
         RGBPixel old_p = m_prev_img.pixel( y, x );

         RGBPixel avgpixel = p.add( old_p, 0.5 );

         m_avg_img.setPixel( y, x, avgpixel );
      }
   }
   findColoredRegion( &m_avg_img, &rect );
   broadcastImageProcessed( &m_avg_img, &rect );
   */
   
   findColoredRegion( img, &rect );
   broadcastImageProcessed( img, &rect );
   //broadcastImageProcessed( img, NULL );

   //xorImages( img );
   //broadcastImageProcessed( &m_xor_img, NULL );

   if( m_prev_img.width() != img->width() )
      m_prev_img.create( img->width(), img->height() );
   img->copyToImage( m_prev_img );

}

void CTImgProc::xorImages( Image *img )
{
   if( m_prev_img.width() != img->width() )
      m_prev_img.create( img->width(), img->height() );

   if( m_xor_img.width() != img->width() )
      m_xor_img.create( img->width(), img->height() );

   if( m_blur_img.width() != img->width() )
      m_blur_img.create( img->width(), img->height() );

   applyBoxFilter( &m_blur_filter, img, &m_blur_img );

   m_blur_img.xorWithImage( &m_prev_img, &m_xor_img );

   m_blur_img.copyToImage( m_prev_img );

   //blurred.copyToImage( *img );
}

void CTImgProc::findColoredRegion( Image* img, Rect* rect)
{
   if( img == NULL )
      return;

   uint w = img->width();
   uint h = img->height();

   int best_x = 0, best_y = 0;
   float best_match = WORST_COLOR_MATCH;
   //Rgb avg_color;
   Hsv avg_hsv;
   Rgb rgb;
   RGBPixel pixel;

   int window_step_size = m_window_size / 2;
   if( window_step_size == 0 )
      window_step_size = 1;

   //printf("----------- New Frame -----------------\n");
   for( uint x = 0; (x + m_window_size) < w; x += window_step_size )
   {
      for( uint y = 0; (y + m_window_size) < h; y += window_step_size )
      {
         //printf("x = %3u, y = %3u\t", x, y);
         //calcAvgColor( img, x, y, avg_color );
         //calcAvgHSV( img, x, y, avg_hsv );

         pixel = img->pixel( y, x );

         rgb.r = pixel.r / 255.0;
         rgb.g = pixel.g / 255.0;
         rgb.b = pixel.b / 255.0;

         avg_hsv = rgb.toHSV();

         
         float color_diff = rgb.calcNormOfDiff( m_target_color );
         //float color_diff = avg_hsv.calcNormOfHSDiff( m_target_hsv );

         //printf("diff = %6.2f\n", color_diff );
         
         if( color_diff < best_match )
         {
            best_match = color_diff;
            best_x = (int) x;
            best_y = (int) y;
         }
      }
   }

   //printf("Best = (%u, %u)\n", best_x, best_y);
      
   rect->x1 = best_x - 1;
   rect->y1 = best_y - 1;
   rect->x2 = best_x + m_window_size;
   rect->y2 = best_y + m_window_size;
}


void CTImgProc::broadcastImageProcessed( Image* img, Rect* rect )
{
   list<CTImgProcListener*>::iterator ipl;
   for( ipl = m_iplisteners.begin(); ipl != m_iplisteners.end(); ipl++ )
   {
      (*ipl)->imageProcessed( img, rect );
   }
}

void CTImgProc::calcAvgColor( Image* img, uint x, uint y, Rgb& avg_color )
{
   uint r_total = 0, g_total = 0, b_total = 0;
   uint s, t;
   RGBPixel pixel;

   for( s = 0; s < m_window_size; s++ )
   {
      for( t = 0; t < m_window_size; t++ )
      {
         pixel = img->pixel( y + t, x + s );

         r_total += pixel.r;
         g_total += pixel.g;
         b_total += pixel.b;
      }
   }

   int pixel_cnt = m_window_size * m_window_size;

   avg_color.r = ((float)r_total / (float)pixel_cnt) / 255.0;
   avg_color.g = ((float)g_total / (float)pixel_cnt) / 255.0;
   avg_color.b = ((float)b_total / (float)pixel_cnt) / 255.0;

   //printf("avg color: %0.2f %0.2f %0.2f\t", avg_color.r, avg_color.g, avg_color.b);
}

void CTImgProc::calcAvgHSV( Image* img, uint x, uint y, Hsv& avg_hsv )
{
   uint s, t;
   RGBPixel pixel;
   Rgb rgb;
   Hsv hsv;

   for( s = 0; s < m_window_size; s++ )
   {
      for( t = 0; t < m_window_size; t++ )
      {
         pixel = img->pixel( y + t, x + s );

         rgb.r = pixel.r / 255.0;
         rgb.g = pixel.g / 255.0;
         rgb.b = pixel.b / 255.0;

         hsv = rgb.toHSV();
         avg_hsv.h += hsv.h;
         avg_hsv.s += hsv.s;
         avg_hsv.v += hsv.v;
      }
   }

   int pixel_cnt = m_window_size * m_window_size;

   avg_hsv.h = avg_hsv.h / pixel_cnt;
   avg_hsv.s = avg_hsv.s / pixel_cnt;
   avg_hsv.v = avg_hsv.v / pixel_cnt;
}

void CTImgProc::applyBoxFilter( BoxFilter* filter, Image* src, Image* dst )
{
   int w = (int) src->width();
   int h = (int) src->height();

   Rgb new_color;

   for( int row = 0; row < h; row++ )
   {
      for( int col = 0; col < w; col++ )
      {
         new_color.r = 0;
         new_color.g = 0;
         new_color.b = 0;

         int src_row = row - filter->size() / 2;
         for( int s = 0; s < filter->size(); s++ )
         {
            if( src_row >= 0 && src_row < h )
            {
               int src_col = col - filter->size() / 2;
               for( int t = 0; t < filter->size(); t++ )
               {
                  if( src_col >= 0 && src_col < w )
                  {
                     RGBPixel src_pixel = src->pixel( src_row, src_col );
                     
                     new_color.r += (*filter)[s][t] * src_pixel.r;
                     new_color.g += (*filter)[s][t] * src_pixel.g;
                     new_color.b += (*filter)[s][t] * src_pixel.b;
                  }

                  src_col++;
               }
            }

            src_row++;
         }

         //printf("dst pixel: %d, %d (%.2f, %.2f, %.2f)\n", row, col,
         //       new_color.r, new_color.g, new_color.b );

         RGBPixel dst_pixel;
         dst_pixel.r = (uchar)(new_color.r);
         dst_pixel.g = (uchar)(new_color.g);
         dst_pixel.b = (uchar)(new_color.b);
         dst->setPixel( row, col, dst_pixel );         

      }
   }

}

