//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////

#ifndef CTIMGPROC_H
#define CTIMGPROC_H


#include "framelistener.h"
#include "image.h"
#include "boxfilter.h"

#include <list>
using std::list;

struct Rect
{
      int x1, y1, x2, y2;
};

class CTImgProcListener
{
   public:
      virtual void imageProcessed( Image* img, Rect* rect ) = 0;
};

class CTImgProc : public FrameListener
{
      
   public:
      CTImgProc();
      virtual ~CTImgProc();

      virtual void frameGrabbed( Image* img );

      inline void addIPListener( CTImgProcListener* l );

      void setTargetPixelColor( const RGBPixel& rgbpixel );

   private:
      class Hsv
      {
         public:
            inline float calcNormOfHSDiff( const Hsv& c );

         public:
            float h, s, v;
      };
      
      class Rgb
      {
         public:
            inline float calcNormOfDiff( const Rgb& c );

            Hsv toHSV();

         public:
            float r, g, b;
      };

   private:
      void calcAvgColor( Image* img, uint x, uint y, Rgb& avg_color );
      void calcAvgHSV( Image* img, uint x, uint y, Hsv& avg_color );
      void broadcastImageProcessed( Image* img, Rect* rect );

      void findColoredRegion( Image* img, Rect* rect);
      void xorImages( Image *img );

      void applyBoxFilter( BoxFilter* filter, Image* src, Image* dst );

   private:
      uint      m_window_size;

      Rgb       m_target_color;

      Hsv       m_target_hsv;

      BlurFilter        m_blur_filter;
      
      list<CTImgProcListener*>  m_iplisteners;

      Image     m_prev_img;
      Image     m_xor_img;
      Image     m_blur_img;
      Image     m_avg_img;
};

void CTImgProc::addIPListener( CTImgProcListener* l )
{
   m_iplisteners.push_back( l );
}


float CTImgProc::Rgb::calcNormOfDiff( const Rgb& c )
{
   return ( (r - c.r) * (r - c.r) +
            (g - c.g) * (g - c.g) +
            (b - c.b) * (b - c.b) );
}

float CTImgProc::Hsv::calcNormOfHSDiff( const Hsv& c )
{
   return ( (h - c.h) * (h - c.h) +
            (s - c.s) * (s - c.s) );
}

#endif
