//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <unistd.h>
#include <semaphore.h>
#include <memory.h>
#include <stdio.h>

#include "framegrabber.h"


#define DEFAULT_CAPTURE_RATE    33


FrameGrabber::FrameGrabber() : m_frl_list()
{
   m_camera = NULL;
   m_image = NULL;
   m_capture_rate = DEFAULT_CAPTURE_RATE;
}

FrameGrabber::~FrameGrabber()
{
   if( m_image != NULL )
      delete m_image;
}


void FrameGrabber::setCamera( V4LCamera* camera )
{
   m_camera = camera;
}

void FrameGrabber::createCaptureThread()
{
   pthread_create( &m_capture_thread, NULL, start_capture, (void*) this );
}

void FrameGrabber::runCaptureThread()
{
   while( !m_shutdown_requested )
   {
      sem_wait( &m_frame_wait_sem );

      if( !m_shutdown_requested )
      {
         m_camera->captureToImage( m_image );

         broadcastCapture();
      }
   }
}

void FrameGrabber::shutdown()
{
   m_shutdown_requested = true;
   m_timer.stop();
}

void FrameGrabber::run()
{
   if( m_camera && m_camera->isOpen() )
   {
      if( m_image != NULL )
         delete m_image;

      m_image = m_camera->createImage();

      m_shutdown_requested = false;

      sem_init( &m_frame_wait_sem, 0, 0 );

      createCaptureThread();

      createTimer();

   }

}

void FrameGrabber::handleTimer()
{
   sem_post( &m_frame_wait_sem );
}

void FrameGrabber::destroyTimer()
{
}

void FrameGrabber::createTimer()
{
   Timer::Period period;
   period.sec = 0;
   period.usec = m_capture_rate * 1000;
   m_timer.create( &period, timer_handler, (void*) this );

   m_timer.start();
}

void FrameGrabber::broadcastCapture()
{
   list<FrameListener*>::iterator frl;

   for( frl = m_frl_list.begin(); frl != m_frl_list.end(); frl++ )
      (*frl)->frameGrabbed( m_image );
}



void timer_handler( void* frame_grabber )
{
   FrameGrabber* fg = (FrameGrabber*) frame_grabber;

   fg->handleTimer();
}

void* start_capture( void* fg_ptr )
{
   FrameGrabber* fg = (FrameGrabber*) fg_ptr;

   fg->runCaptureThread();

   return NULL;
}
