//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////

#ifndef FRAMEGRABBER_H
#define FRAMEGRABBER_H

#include <semaphore.h>
#include <pthread.h>

#include "image.h"
#include "v4lcamera.h"
#include "framelistener.h"
#include "timer.h"

#include <list>
using std::list;

class FrameGrabber
{
   public:
      FrameGrabber();
      ~FrameGrabber();

      inline void addFrameListener( FrameListener* l );
      
      void setCamera( V4LCamera* camera );
      
      void run();

      void shutdown();

   private:
      void broadcastCapture();

      void createTimer();
      void destroyTimer();
      void handleTimer();

      void createCaptureThread();
      void runCaptureThread();
      
      void destroyFLList();
      
      friend void timer_handler( void* frame_grabber );
      friend void* start_capture( void* fg_ptr );

   private:
      
   private:
      V4LCamera*        m_camera;

      Image*            m_image;

      int               m_capture_rate;
      
      timer_t           m_timer_id;
      
      sem_t             m_frame_wait_sem;

      Timer             m_timer;
      pthread_t         m_capture_thread;
      
      list<FrameListener*>      m_frl_list;

      bool              m_shutdown_requested;
};


void FrameGrabber::addFrameListener( FrameListener* frl )
{
   m_frl_list.push_back( frl );
}


#endif
