//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////

#ifndef IMAGE_H
#define IMAGE_H

#include "types.h"


class HSVPixel
{
   public:
      uchar     h;
      uchar     s;
      uchar     v;
};

class RGBPixel
{
   public:
      RGBPixel xorWith(const RGBPixel& pixel);

      HSVPixel toHSVPixel();

      RGBPixel add(const RGBPixel& pixel, float alpha);

      void setRGB( uchar _r, uchar _g, uchar _b );
      
   public:
      uchar     r;
      uchar     g;
      uchar     b;
};

class Image
{
   public:
      Image();
      ~Image();

      void copyToImage( Image& img );
      
      void create(uint w, uint h);
      void copyFromPixArray(uchar* pixels, uint w, uint h);

      bool loadFromJPEG(const char* filename);

      RGBPixel pixel( int row, int col );

      inline RGBPixel* scanLine( int row );

//      inline uchar* getPixels();

      void setPixel( int row, int col, const RGBPixel& pixel );
      inline void setPixel( int pixel_ind, const RGBPixel& pixel );

      inline unsigned int width();
      inline unsigned int height();


      void xorWithImage( Image* src, Image* dst );
      
   private:
      RGBPixel* m_pixels;
      uint      m_w;
      uint      m_h;
};


RGBPixel* Image::scanLine( int row )
{
   return &m_pixels[ row * m_w ];
}

void Image::setPixel( int pixel_ind, const RGBPixel& pixel )
{
   m_pixels[ pixel_ind ] = pixel;
}

/*
uchar* Image::getPixels()
{
   return m_pixels;
}
*/

unsigned int Image::width()
{
   return m_w;
}

unsigned int Image::height()
{
   return m_h;
}

#endif
