//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////


#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qvgroupbox.h>
#include <qhbox.h>
#include <qgrid.h>

#include "imagepropdlg.h"
#include "v4lcamera.h"

ImagePropDlg::ImagePropDlg( QWidget* parent, V4LCamera* camera ) :
   QDialog( parent, "ImageProps", TRUE )
{
   m_camera = camera;

   bool valid_camera = false;
   
   if( m_camera != NULL && m_camera->isOpen() )
   {
      valid_camera = true;
   }
   
   setCaption( "Image Properties" );

   main_vbox = new QVBoxLayout( this, 5, 4, "main_vbox" );

   QGroupBox* gbox;

   gbox = new QVGroupBox( "Image Properties", this, "gbox" );

   QGrid* grid = new QGrid( 2, gbox );
   grid->setSpacing( 4 );


   createSliders( gbox );

   QPushButton* close_button = new QPushButton( "Close", this );

   connect( close_button, SIGNAL( clicked() ), this, SLOT( accept() ) );

   main_vbox->addWidget( gbox );
   main_vbox->addWidget( close_button );

}

ImagePropDlg::~ImagePropDlg()
{
   //delete grid;
   //delete main_vbox;
}

void ImagePropDlg::setImageProps( int value )
{
   V4LCamera::ImgProps imgprops;
   m_camera->getImgProps( &imgprops );

   imgprops.brightness = m_brightness->value();
   imgprops.hue = m_hue->value();
   imgprops.color = m_color->value();
   imgprops.contrast = m_contrast->value();

   m_camera->setImgProps( &imgprops );
}


void ImagePropDlg::createSliders( QWidget* parent )
{
   V4LCamera::ImgProps imgprops;

   m_camera->getImgProps( &imgprops );

   QLabel* l;
   
   l = new QLabel( "Brightness", parent );
   m_brightness = new QSlider( 0,
                               65535,
                               1,
                               imgprops.brightness,
                               Qt::Horizontal,
                               parent,
                               "brightness slider" );
   m_brightness->setTracking( false );
   connect( m_brightness, SIGNAL( valueChanged(int) ),
            this, SLOT( setImageProps(int) ) );


   l = new QLabel( "Hue", parent );
   m_hue = new QSlider( 0,
                        65535,
                        1,
                        imgprops.hue,
                        Qt::Horizontal,
                        parent,
                        "hue slider" );
   m_hue->setTracking( false );
   connect( m_hue, SIGNAL( valueChanged(int) ),
            this, SLOT( setImageProps(int) ) );


   l = new QLabel( "Color", parent );
   m_color = new QSlider( 0,
                          65535,
                          1,
                          imgprops.color,
                          Qt::Horizontal,
                          parent,
                          "color slider" );
   m_color->setTracking( false );
   connect( m_color, SIGNAL( valueChanged(int) ),
            this, SLOT( setImageProps(int) ) );


   l = new QLabel( "Contrast", parent );
   m_contrast = new QSlider( 0,
                             65535,
                             1,
                             imgprops.contrast,
                             Qt::Horizontal,
                             parent,
                             "contrast slider" );
   m_contrast->setTracking( false );
   connect( m_contrast, SIGNAL( valueChanged(int) ),
            this, SLOT( setImageProps(int) ) );

}

