//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////



#include <qapplication.h>

#include "appwindow.h"
#include "framegrabber.h"
#include "ctimgproc.h"
#include "ctcontroller.h"
#include "appdata.h"

extern AppData gAppData;


int initCamera( const char* camera_device )
{
   V4LCamera* cam = gAppData.getCamera();

   if( !cam->openDevice( camera_device ) )
   {
      fprintf( stderr, "Could not open camera on %s\n", camera_device );
      return 0;
   }

   return 1;
}

void shutdownCamera()
{
   V4LCamera* cam = gAppData.getCamera();
   cam->closeDevice();
}

void startFrameGrabber( FrameGrabber* fg )
{
   fg->setCamera( gAppData.getCamera() );
   fg->run();
}

int main(int argc, char** argv)
{
   QApplication app( argc, argv );

   FrameGrabber frame_grabber;

   AppWindow app_window;

   CTImgProc img_processor;

   CTController controller;

   const char* camera_device = "/dev/video1";
   if( argc > 1 )
      camera_device = argv[1];

   if( !initCamera( camera_device ) )
   {
      return -1;
   }

   startFrameGrabber( &frame_grabber );

   app.setMainWidget( &app_window );

   app_window.setController( &controller );
   app_window.setImgProc( &img_processor );
   
   app_window.show();

   frame_grabber.addFrameListener( &img_processor );

   img_processor.addIPListener( &app_window );
   img_processor.addIPListener( &controller );

   int ret = app.exec();

   frame_grabber.shutdown();
   
   shutdownCamera();
   
   return ret;
}
