//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////

#include "timer.h"

#include <unistd.h>
#include <pthread.h>
#include <sys/time.h>

Timer::Timer()
{
   m_callback = NULL;
   m_running = false;
}

void Timer::create( Period* period, void (*callback)(void*), void* arg )
{
   m_period.sec = period->sec;
   m_period.usec = period->usec;
   m_callback = callback;
   m_arg = arg;
}

void Timer::start()
{
   m_running = true;

   pthread_t tid;
   pthread_create( &tid, NULL, timer_thread_wrap, (void*) this );
}

void Timer::stop()
{
   m_running = false;
}

void Timer::runTimerThread()
{
   struct timeval tv;
   struct timezone tz;
   int total_usec;
   int time_before;
   int time_after;

   total_usec = m_period.sec * 1000000 + m_period.usec;

   usleep( total_usec );

   while( m_running )
   {
      gettimeofday( &tv, &tz );
      time_before = (int)(tv.tv_sec * 1000000) + (int)(tv.tv_usec);

      m_callback( m_arg );

      gettimeofday( &tv, &tz );
      time_after = (int)(tv.tv_sec * 1000000) + (int)(tv.tv_usec);

      int time_tosleep = total_usec - (time_after - time_before);
      if( time_tosleep > 0 )
         usleep( time_tosleep );
   }
}

void* timer_thread_wrap( void* timer )
{
   Timer* t = (Timer*) timer;
   t->runTimerThread();
}

