//////////////////////////////////////////////////////////////////
// Single Author Info
//   dehuber  Daniel E. Huber
//
// Group Info
//   dehuber  Daniel E. Huber
//   jjwinega Joel J. Winegarden
//////////////////////////////////////////////////////////////////

#ifndef V4LCAMERA_H
#define V4LCAMERA_H

#include <linux/videodev.h>

#include "image.h"

class V4LCamera
{
   public:
      struct CamInfo
      {
            const char*         name;
            const char*         device_name;
            int                 type;
            int                 min_width;
            int                 max_width;
            int                 min_height;
            int                 max_height;
            int                 x;
            int                 y;
            int                 width;
            int                 height;
            int                 depth;
            bool                color;
      };

      enum
      {
         PALETTE_GREY,
         PALETTE_HI240,
         PALETTE_RGB565,
         PALETTE_RGB555,
         PALETTE_RGB24,
         PALETTE_RGB32,
         PALETTE_YUV422,
         PALETTE_YUYV,
         PALETTE_UYVY,
         PALETTE_YUV420,
         PALETTE_YUV411,
         PALETTE_RAW,
         PALETTE_YUV422P,
         PALETTE_YUV411P,
      };
      
      struct ImgProps
      {
            ushort      brightness;
            ushort      hue;
            ushort      color;
            ushort      contrast;
            ushort      depth;
            ushort      palette;

            bool        adaptive;
      };

      enum
      {
         OK,
         INVALID_DIMS,
      };
      
   public:
      V4LCamera();
      ~V4LCamera();

      bool openDevice( const char* device_name );
      void closeDevice();

      void getInfo( CamInfo* caminfo );

      void getImgProps( ImgProps* imgprops );
      void setImgProps( ImgProps* imgprops );
      
      void prepareCapture();
      
      int captureToImage( Image* img );

      Image* createImage();
      
      inline bool isOpen();
      
   protected:
      int               device_fd;
      const char*       m_device_name;

      video_capability  vidcap;
      video_window      vidwin;
      video_picture     vidpic;
      video_clip        vidclips[32];

      uchar*            m_pixels_mmap;
};


bool V4LCamera::isOpen()
{
   return device_fd;
}


#endif
